"""<name>Molecule Visualizer</name>
<description>Rendering of 2D structure of molecules based on their SMILES description.</description>
<icon>icons/MoleculeVisualizer.png</icon>
<contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si)</contact> 
<priority>2050</priority>
"""

from __future__ import absolute_import, with_statement

import sys, os, urllib2, urllib
import warnings
import shelve
from cStringIO import StringIO
import pickle

from PyQt4.QtSvg import *

import orange
from Orange.orng import orngEnviron
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import obiChem

class dummy_module(object):
	def __init__(self, name):
		self.name = name
	def __getattr__(self, name):
		raise ImportError(self.name)
	def __nonzero__(self):
		return False

try:
	import pybel
except ImportError:
	pybel = dummy_module("pybel")
except Exception, ex:
	pybel = dummy_module("pybel")

svg_error_string = """<?xml version="1.0" ?>
<svg height="185" version="1.0" width="250" xmlns="http://www.w3.org/2000/svg">
	<g stroke="#000" stroke-width="1.0">
		<text font-family="Arial" font-size="16" stroke="rgb(0, 0, 0)" x="0.0" y="16.0">
			%s
		</text>
	</g>
</svg>
"""

oasaLocal = True

#  try import from bkchem 
try:
    from bkchem import oasa
except ImportError:
    #  try stanalone import 
	try:
	    import oasa
	except ImportError:
	    oasaLocal = False

if oasaLocal and pybel:
    def mol_to_svg(molSmiles, fragSmiles):
        s = StringIO()
        obiChem.mol_to_svg(molSmiles, fragSmiles, s)
        s.seek(0)
        return s.read()
else:
    def mol_to_svg(molSmiles, fragSmiles):
        params = urllib.urlencode({'molSmiles': molSmiles, 'fragSmiles': fragSmiles})
        f = urllib2.urlopen("http://asterix.fri.uni-lj.si/misc/bkchem/drawMol_oasa.py", params)
        return f.read()
  
class DrawContext(object):
    def __init__(self, molecule="", fragment="", size=200, imageprefix="", imagename="", title="", grayedBackground=False, useCached=False):
        self.molecule=molecule
        self.fragment=fragment
        self.size=size
        self.imageprefix=imageprefix
        self.imagename=imagename
        self.title=title
        self.grayedBackground=grayedBackground
        self.useCached=useCached

    def __hash__(self):
        return (self.molecule+"%%"+self.fragment+"%%"+str(self.size)+"%%"+self.title+str(self.grayedBackground)).__hash__()

    def __eq__(self, other):
        return self.molecule==other.molecule and self.fragment==other.fragment and \
               self.size==other.size and self.title==other.title and self.grayedBackground==other.grayedBackground

    def __ne__(self, other):
        return not self==other
            
from threading import RLock

def synchronized(lock):
    def syncfunc(func):
        def f(*args, **kw):
            with lock:
                ret = func(*args, **kw)
            return ret
        return f
    return syncfunc

class ImageCache(object):
    __shared_state = {"shelve": None}
    lock = RLock()
    @synchronized(lock)
    def __init__(self):
        self.__dict__ = self.__shared_state
        if self.shelve is None:
            try:
                os.mkdir(os.path.join(orngEnviron.bufferDir, "molimages"))
            except OSError:
                pass
            try:
                self.shelve = shelve.open(os.path.join(orngEnviron.bufferDir, "molimages", "cache.shelve"))
            except Exception, ex:
                warnings.warn("Cannot open molecule images cache! " + str(ex))
                self.shelve = {}
    
    
    @synchronized(lock)
    def __getitem__(self, key):
        val_str = str(key)
        if val_str in self.shelve:
            return self.shelve[val_str]
        else:
            mol_smiles = key[0]
            frag_smiles = key[1] if len(key) > 1 else None
            self.shelve[val_str] = mol_to_svg(mol_smiles, frag_smiles)
            return self.shelve[val_str]

    @synchronized(lock)
    def __setitem__(self, key, value):
        if len(self.shelve) > 1000:
            self.sync()
        self.shelve[str(key)] = value

            
    @synchronized(lock)
    def __contains__(self, key):
        return str(key) in self.shelve
    
    @synchronized(lock)
    def sync(self):
        if len(self.shelve.keys()) > 1000:
            for key in self.shelve.keys()[:-900]:
                del self.shelve[key]
        if hasattr(self.shelve, "sync"):
            self.shelve.sync()

    def __del__(self):
    	if hasattr(self.shelve, " sync"):
            self.sync()

class MolWidget(QFrame):
    def setSelected(self, val):
        self.image.selected = val
    def getSelected(self):
        return self.image.selected
    selected = property(getSelected, setSelected)

    def __init__(self, master, parent, context):
        QFrame.__init__(self, parent)
        self.master=master
        self.context=context
        self.label=QLabel()
        self.label.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        try:
        	self.from_cache = (context.molecule, context.fragment) in ImageCache()
        	s = ImageCache()[context.molecule, context.fragment]
        	self.state = 1
        except Exception, ex:
            from traceback import print_exc
            print_exc()
            s = svg_error_string % "Error loading: "+str(ex)
            self.from_cache = False
            self.state = 0
        self.image=SVGImageThumbnail(s, self)
        self.image.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.label.setText(context.title)
        self.label.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.label.setMaximumWidth(context.size)
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        layout.addWidget(self.image)
        self.setLayout(layout)
        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.show()

    def repaint(self):
        QFrame.repaint(self)
        self.label.repaint()
        self.image.repaint()
        
class SVGImageThumbnail(QFrame):
    def setSelected(self, val):
        self._selected = val
        self.update()
    def getSelected(self):
        return self._selected
    selected = property(getSelected, setSelected)
    
    @property
    def highlight(self):
        return self.parent().context.grayedBackground
    
    def __init__(self, file, parent):
        QWidget.__init__(self, parent)
        self.doc = file if type(file) == str else file.read()
        self.renderer = QSvgRenderer(QByteArray(self.doc))
        self.buffer = None
        self.resize(200, 200)
        self.selected = False
        
    def paintEvent(self, event):
        if not self.buffer or self.buffer.size() != self.size():
            defSize = self.renderer.defaultSize()
            scaleFactor = float(self.width()) / max(defSize.width(), defSize.height())
            self.buffer = QImage(self.size(), QImage.Format_ARGB32_Premultiplied)
            self.buffer.fill(0)
            painter = QPainter(self.buffer)
            painter.setViewport(self.width()/2 - scaleFactor*defSize.width()/2, self.height()/2 - scaleFactor*defSize.height()/2,
                                scaleFactor*defSize.width(), scaleFactor*defSize.height())
            self.renderer.render(painter)
        painter = QPainter(self)            
        painter.setBrush(QBrush(Qt.white))
        painter.drawRect(0, 0, self.width(), self.height())
        if self.selected:
            painter.setPen(QPen(QBrush(Qt.red), 2))
        if self.highlight:
            painter.setBrush(QBrush(Qt.gray,  Qt.FDiagPattern))
        else:
            painter.setBrush(Qt.NoBrush)
        painter.drawRect(0, 0, self.width(), self.height())
        painter.drawImage(0, 0, self.buffer)

    def sizeHint(self):
        return QSize(self.parent().master.imageSize, self.parent().master.imageSize)

    def mouseDoubleClickEvent(self, event):
        self._bigimage = BigSvgWidget()
        self._bigimage.load(QByteArray(self.doc))
        self._bigimage.show()

    def mousePressEvent(self, event):
        self.parent().master.mouseAction(self.parent(), event)

class BigSvgWidget(QSvgWidget):
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setBrush(QBrush(Qt.white))
        painter.drawRect(0, 0, self.width(), self.height())
        painter.end()
        QSvgWidget.paintEvent(self, event)
        
class ScrollArea(QScrollArea):
    def __init__(self, master, *args):
        QScrollArea.__init__(self, *args)
        self.master=master
        self.viewport().setMouseTracking(True)
        self.setMouseTracking(True)
        
    def resizeEvent(self, event):
        QScrollArea.resizeEvent(self, event)
        size = event.size()
        w, h=self.width(), self.height()
        oldNumColumns = self.master.numColumns
        numColumns = w / (self.master.imageSize + self.master.gridLayout.horizontalSpacing()*2 + 20) or 1
        if numColumns != oldNumColumns:
            self.master.numColumns = numColumns
            self.master.rearrangeLayout()

class OWMoleculeVisualizer(OWWidget):
    settingsList=["colorFragmets","showFragments"]
    contextHandlers={"":DomainContextHandler("", [ContextField("moleculeTitleAttributeList",
                                                    DomainContextHandler.List + DomainContextHandler.SelectedRequired + DomainContextHandler.IncludeMetaAttributes,
                                                    selected="selectedMoleculeTitleAttrs"),
                                                  ContextField("moleculeSmilesAttr", DomainContextHandler.Required + DomainContextHandler.IncludeMetaAttributes)], maxAttributesToPickle=10000)} ##maxAttributesToPickle=10000 some bug in Context handler 
    def __init__(self, parent=None, signalManager=None, name="Molecule visualizer"):
        OWWidget.__init__(self, parent, signalManager, name)
        self.inputs=[("Molecules", ExampleTable, self.setMoleculeTable), ("Molecule subset", ExampleTable, self.setMoleculeSubset), ("Fragments", ExampleTable, self.setFragmentTable)]
        self.outputs=[("Selected Molecules", ExampleTable)]
        self.colorFragments=1
        self.showFragments=0
        self.selectedFragment=""
        self.moleculeSmiles=[]
        self.fragmentSmiles=[]
        self.defFragmentSmiles=[]
        self.moleculeSmilesAttr=0
        self.moleculeTitleAttr=0
        self.moleculeTitleAttributeList=[]
        self.selectedMoleculeTitleAttrs=[]
        self.fragmentSmilesAttr=0
        self.imageSize=200
        self.numColumns=4
        self.commitOnChange=0
        self.overRideCache=True
        ##GUI
        box=OWGUI.widgetBox(self.controlArea, "Info", addSpace = True)
        self.infoLabel=OWGUI.label(box, self, "Chemicals: ")
##        if not oasaLocal:
##            OWGUI.label(box, self, "OpenEye not installed, access to server required.")
##            self.serverInfo = OWGUI.label(box, self, "")
        box=OWGUI.radioButtonsInBox(self.controlArea, self, "showFragments", ["Show molecules", "Show fragments"], "Show", callback=self.showImages)
        self.showFragmentsRadioButton=box.buttons[-1]
        self.markFragmentsCheckBox=OWGUI.checkBox(box, self, "colorFragments", "Mark fragments", callback=self.redrawImages)
        box.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
        OWGUI.separator(self.controlArea)
        self.moleculeSmilesCombo=OWGUI.comboBox(self.controlArea, self, "moleculeSmilesAttr", "Molecule SMILES Attribute",callback=self.showImages)
        self.moleculeSmilesCombo.box.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
        OWGUI.separator(self.controlArea)
##        self.moleculeTitleCombo=OWGUI.comboBox(self.controlArea, self, "moleculeTitleAttr", "Molecule title attribute", callback=self.redrawImages)
        box=OWGUI.widgetBox(self.controlArea, "Molecule Title Attributes", addSpace = True)
##        self.moleculeTitleListBox=QListBox(box)
##        self.moleculeTitleListBox.setSelectionMode(QListBox.Extended)
##        self.moleculeTitleListBox.setMinimumHeight(100)
##        self.connect(self.moleculeTitleListBox, SIGNAL("selectionChanged()"), self.updateTitles)
        self.moleculeTitleListBox=OWGUI.listBox(box, self, "selectedMoleculeTitleAttrs", "moleculeTitleAttributeList", selectionMode = QListWidget.ExtendedSelection, callback=self.updateTitles)
        self.moleculeTitleListBox.setMinimumHeight(100)
##        OWGUI.separator(self.controlArea)
##        self.moleculeTitleCombo.box.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
        OWGUI.separator(self.controlArea)
        self.fragmentSmilesCombo=OWGUI.comboBox(self.controlArea, self, "fragmentSmilesAttr", "Fragment SMILES Attribute", callback=self.updateFragmentsListBox)
        self.fragmentSmilesCombo.box.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
        OWGUI.separator(self.controlArea)
        box=OWGUI.spin(self.controlArea, self, "imageSize", 50, 500, 50, box="Image Size", callback=self.redrawImages, callbackOnReturn = True)
        box.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum))
        OWGUI.separator(self.controlArea)
        box=OWGUI.widgetBox(self.controlArea,"Selection", addSpace = True)
        OWGUI.checkBox(box, self, "commitOnChange", "Commit on change")
        self.selectMarkedMoleculesButton=OWGUI.button(box, self, "Select &matched molecules", self.selectMarked)
        OWGUI.button(box, self, "&Commit", callback=self.commit)
        OWGUI.separator(self.controlArea)
        OWGUI.button(self.controlArea, self, "&Save to HTML", self.saveToHTML, debuggingEnabled = 0)
        OWGUI.rubber(self.controlArea)
        
        spliter=QSplitter(Qt.Vertical)
        self.scrollArea=ScrollArea(self, spliter)
        
        self.molWidget=QWidget()
        self.scrollArea.setWidget(self.molWidget)
        self.mainArea.layout().addWidget(spliter)
        self.gridLayout=QGridLayout(self.molWidget)
        self.molWidget.setLayout(self.gridLayout)

        if pybel:
        	self.listBox=QListWidget(spliter)
        else:
        	self.listBox=QListWidget(None)
        	self.listBox.setHidden(True)

        self.connect(self.listBox, SIGNAL("itemClicked(QListWidgetItem *)"), self.fragmentSelection)
        
        self.fragmentSmilesCombo.box.setDisabled(not pybel)

        self.imageWidgets=[]
        self.candidateMolSmilesAttr=[]
        self.candidateMolTitleAttr=[None]
        self.candidateFragSmilesAttr=[None]
        self.molData=None
        self.molSubset=[]
        self.fragData=None
        self.ctrlPressed=FALSE
        self.resize(800,600)
        self.listBox.setMaximumHeight(150)
        self.fragmentSmilesCombo.setDisabled(True)
        self.selectMarkedMoleculesButton.setDisabled(True)
        self.markFragmentsCheckBox.setDisabled(True)
        self.showFragmentsRadioButton.setDisabled(True)
        self.loadSettings()
        self.failedCount=0
        self.fromCacheCount=0
        
        if not pybel:
        	self.showFragments = 0
        	self.warning(10, "Pybel module not installed. To view molecule fragments\nplease install openbabel python extension.")
        if not oasaLocal:
        	self.warning(11, "OASA module not installed. For faster local molecule\nrendering install OASA module (part of BKChem)")
        
    def setMoleculeTable(self, data):
        self.closeContext()
        self.molData=data
        if data:
            self.setMoleculeSmilesCombo()
            self.setMoleculeTitleListBox()
            self.setFragmentSmilesCombo()
            self.updateFragmentsListBox()
            if self.molSubset:
                try:
                    self.molSubset=self.molSubset.select(self.molData.domain)
                except:
                    self.molSubset=[]
            tmp=self.moleculeTitleAttributeList
            self.openContext("", data)
            if tmp and not self.moleculeTitleAttributeList: ##openContext somtimes crashes internaly and silently clears title list
                self.moleculeTitleAttributeList=tmp
            self.showImages()
        else:
            self.moleculeSmilesCombo.clear()
            self.moleculeTitleListBox.clear()
            self.moleculeSmilesAttr=0
            self.moleculeTitleAttributeList=[]
            self.selectedMoleculeTitleAttrs=[]
            self.defFragmentSmiles=[]
            if not self.fragmentSmilesAttr:
                self.listBox.clear()
            self.destroyImageWidgets()
            self.openContext("",data)
            self.send("Selected Molecules", None)
    
    def setMoleculeSubset(self, data):
        self.molSubset=data
        try:
            self.molSubset=self.molSubset.select(self.molData.domain)
        except:
            self.molSubset=[]
        self.showImages()
            
    def setFragmentTable(self, data):
        self.fragData=data
        if data:
            self.setFragmentSmilesCombo()
            self.updateFragmentsListBox()
            self.selectedFragment=""
            self.showImages()
        else:
            self.setFragmentSmilesCombo()
            #self.fragmentSmilesAttr=0
            self.updateFragmentsListBox()
            if self.showFragments:
                self.destroyImageWidgets()
        self.fragmentSmilesCombo.setDisabled(bool(data))

    def filterSmilesVariables(self, data):
#    	import pybel
        candidates = data.domain.variables+data.domain.getmetas().values()
        candidates = filter(lambda v:v.varType==orange.VarTypes.Discrete or v.varType==orange.VarTypes.String, candidates)
        if len(data)>20:
            data=data.select(orange.MakeRandomIndices2(data, 20))
        vars=[]
        def isValidSmiles(s):
            try:
                pybel.readstring("smi", s)
            except IOError:
                return False
            except ImportError:
            	return True
            return True
            
        import os
        tmpFd1=os.dup(1)
        tmpFd2=os.dup(2)
        fd=os.open(os.devnull, os.O_APPEND)
##        os.close(1)
        os.dup2(fd, 1)
        os.dup2(fd, 2)
##        os.close(fd)
        for var in candidates:
            count=0
            for e in data:
                if pybel and isValidSmiles(str(e[var])):
                    count+=1
            vars.append((count,var))
        names=[v.name for v in data.domain.variables+data.domain.getmetas().values()]
        names=filter(isValidSmiles, names)
##        os.close(1)
        os.dup2(tmpFd1, 1)
        os.dup2(tmpFd2, 2)
##        os.close(tmpFd)
        return vars, names
        
    def setMoleculeSmilesCombo(self):
        candidates, self.defFragmentSmiles=self.filterSmilesVariables(self.molData)
        self.candidateMolSmilesAttr=[c[1] for c in candidates]
        best = reduce(lambda best,current:best[0]<current[0] and current or best, candidates)
        self.moleculeSmilesCombo.clear()
        self.moleculeSmilesCombo.addItems([v.name for v in self.candidateMolSmilesAttr])
        self.moleculeSmilesAttr=candidates.index(best)

    def setMoleculeTitleListBox(self):
        self.icons=self.createAttributeIconDict()
        vars=self.molData.domain.variables+self.molData.domain.getmetas().values()
        self.moleculeTitleAttributeList=[attr.name for attr in vars]
        self.selectedMoleculeTitleAttrs=[]

    def updateTitles(self):
        if not self.molData:
            return
        smilesAttr=self.candidateMolSmilesAttr[min(self.moleculeSmilesAttr, len(self.candidateMolSmilesAttr)-1)]
        attrs = self.molData.domain.variables+self.molData.domain.getmetas().values()
        selected=[attrs[i] for i in self.selectedMoleculeTitleAttrs]
        for widget, example in zip(self.imageWidgets, filter(lambda e:not e[smilesAttr].isSpecial(),self.molData)):
            text=" / ".join(map(str, [example[attr] for attr in selected]))
            widget.label.setText(text)

    def setFragmentSmilesCombo(self):
        if self.fragData:
            candidates, names=self.filterSmilesVariables(self.fragData)
        else:
            candidates=[]
        self.candidateFragSmilesAttr=[None]+candidates
        self.fragmentSmilesCombo.clear()
        self.fragmentSmilesCombo.addItems(["Default"]+[v.name for v in candidates])
        if self.fragmentSmilesAttr>len(candidates):
            self.fragmentSmilesAttr=0

    def updateFragmentsListBox(self):
    	if not pybel:
    		return
        fAttr=self.candidateFragSmilesAttr[self.fragmentSmilesAttr]
        if fAttr:
            self.fragmentSmiles=[""]+[str(e[fAttr]) for e in self.fragData if not e[fAttr].isSpecial()]
        else:
            self.fragmentSmiles=[""]+self.defFragmentSmiles
        self.listBox.clear()
        self.listBox.addItems(self.fragmentSmiles)
        self.showFragmentsRadioButton.setDisabled(len(self.fragmentSmiles)==1)
        self.markFragmentsCheckBox.setDisabled(len(self.fragmentSmiles)==1)
        self.selectMarkedMoleculesButton.setDisabled(True)
        
    def fragmentSelection(self, item):
    	if not pybel:
    		return
        index = self.listBox.indexFromItem(item).row()
        if index == -1:
            index = 0
        self.selectedFragment=self.fragmentSmiles[index]
        self.selectMarkedMoleculesButton.setEnabled(bool(self.selectedFragment))
        self.markFragmentsCheckBox.setEnabled(bool(self.selectedFragment))
        if not self.showFragments and self.colorFragments:
            self.redrawImages()
        
    def renderImages(self,useCached=False):
##        def fixNumColumns(numItems, numColumns):
##            if (self.imageSize+4)*(numItems/numColumns+1)>30000:
##                return numItems/(30000/(self.imageSize+4))
##            else:
##                return numColumns
        
        self.numColumns=self.scrollArea.width()/(self.imageSize+4) or 1
        self.gridLayout = QGridLayout()
        self.scrollArea.takeWidget()
        self.molWidget = QWidget()
        self.scrollArea.setWidget(self.molWidget)
        self.molWidget.setLayout(self.gridLayout)
        self.imageWidgets=[]
##        self.imageCache.newEpoch()
        self.failedCount=0
        self.fromCacheCount=0
        if self.showFragments and self.fragmentSmiles:
            correctedNumColumns=self.numColumns #fixNumColumns(len(self.fragmentSmiles[1:]), self.numColumns)
            self.progressBarInit()
            for i,fragment in enumerate(self.fragmentSmiles[1:]):
                #imagename=self.imageprefix+str(i)+".bmp"
                #vis.molecule2BMP(fragment, imagename, self.imageSize)
##                image=MolImage(self,  self.molWidget, DrawContext(molecule=fragment, imagename=imagename, size=self.imageSize))
                image=MolWidget(self, self.molWidget, DrawContext(molecule=fragment, size=self.imageSize))
                self.gridLayout.addWidget(image, i/correctedNumColumns, i%correctedNumColumns)
                self.imageWidgets.append(image)
                self.progressBarSet(i*100/len(self.fragmentSmiles))
            self.progressBarFinished()
        elif self.molData and self.candidateMolSmilesAttr:
            sAttr=self.candidateMolSmilesAttr[min(self.moleculeSmilesAttr, len(self.candidateMolSmilesAttr)-1)]
            tAttr=self.candidateMolTitleAttr[min(self.moleculeTitleAttr, len(self.candidateMolTitleAttr)-1)]
            if self.moleculeTitleAttr:
                titleList=[str(e[tAttr]) for e in self.molData if not e[sAttr].isSpecial()]
            else:
                titleList=[]
                if not sAttr:
                    return
            molSmiles=[(str(e[sAttr]), e) for e in self.molData if not e[sAttr].isSpecial()]
            correctedNumColumns=self.numColumns #fixNumColumns(len(molSmiles), self.numColumns)
            self.progressBarInit()
            if self.colorFragments and self.selectedFragment:
                fMap=map_fragments([self.selectedFragment], [t[0] for t  in molSmiles])
            for i,((molecule, example), title) in enumerate(zip(molSmiles, titleList or [""]*len(molSmiles))):
                #imagename=self.imageprefix+str(i)+".bmp"
                if self.colorFragments and self.selectedFragment and fMap[molecule][self.selectedFragment]:
                    context=DrawContext(molecule=molecule, fragment=self.selectedFragment, size=self.imageSize, title=title, grayedBackground=example in self.molSubset, useCached=useCached)
                    #vis.moleculeFragment2BMP(molecule, self.selectedFragment, imagename, self.imageSize)
                else:
                    context=DrawContext(molecule=molecule, size=self.imageSize, title=title, grayedBackground=example in self.molSubset, useCached=useCached)
                    #vis.molecule2BMP(molecule, imagename, self.imageSize)
##                image=MolImage(self, self.molWidget, context)
                image=MolWidget(self, self.molWidget, context)
                self.gridLayout.addWidget(image, i/correctedNumColumns, i%correctedNumColumns)
                self.imageWidgets.append(image)
                self.progressBarSet(i*100/len(molSmiles))
            self.progressBarFinished()
            self.updateTitles()
        #print "done drawing"
        self.overRideCache=False
        self.fromCacheCount = sum(int(w.from_cache) for w in self.imageWidgets)
        self.failedCount = len(self.imageWidgets) - sum(int(w.state) for w in self.imageWidgets)
        
        self.molWidget.setMinimumSize(self.gridLayout.sizeHint())
        self.molWidget.show()

    def destroyImageWidgets(self):
        for w in self.imageWidgets:
            w.hide()
            self.gridLayout.removeWidget(w)
        self.imageWidgets=[]

    def rearrangeLayout(self):
        self.numColumns=self.scrollArea.width() / (self.imageSize + self.gridLayout.horizontalSpacing()*2 + 20) or 1
        self.molWidget = QWidget()
        self.gridLayout = QGridLayout()
        self.molWidget.setLayout(self.gridLayout)
        for i, w in enumerate(self.imageWidgets):
            self.gridLayout.addWidget(w, i/self.numColumns, i%self.numColumns)
        self.scrollArea.takeWidget()
        self.scrollArea.setWidget(self.molWidget)
        self.molWidget.setMinimumSize(self.gridLayout.sizeHint())
        self.molWidget.show()
            
    def showImages(self, useCached=False):
        self.destroyImageWidgets()
        self.warning(0)
        
        self.renderImages(useCached)
        if not oasaLocal:
            if self.failedCount>0:
                self.infoLabel.setText("%i chemicals\nFailed to retrieve %i images from server\n%i images from server\n%i images from local cache" \
									   % (len(self.imageWidgets), self.failedCount, len(self.imageWidgets)-self.failedCount-self.fromCacheCount, self.fromCacheCount))
                self.warning(0, "Failed to retrieve some images from server")
            elif self.fromCacheCount == len(self.imageWidgets):
                self.infoLabel.setText("%i chemicals\nAll images from local cache" % len(self.imageWidgets))
            elif self.fromCacheCount == 0:
                self.infoLabel.setText("%i chemicals\nAll images from server" % len(self.imageWidgets))
            else:
                self.infoLabel.setText("%i chemicals\n%i images from server\n%i from local cache" % (len(self.imageWidgets), len(self.imageWidgets)-self.fromCacheCount, self.fromCacheCount))
        else:
            self.infoLabel.setText("Chemicals %i" % len(self.imageWidgets))

    def redrawImages(self, useCached=False):
        selected=map(lambda i:self.imageWidgets.index(i), filter(lambda i:i.selected, self.imageWidgets))
        self.showImages(useCached=useCached)
        for i in selected:
            self.imageWidgets[i].selected=True
            self.imageWidgets[i].repaint()
            
    def mouseAction(self, image, event):
        if self.ctrlPressed:
            image.selected=not image.selected
        else:
            for i in self.imageWidgets:
                i.selected=False
                i.repaint()
            image.selected=True
        image.repaint()
        if self.commitOnChange:
            self.commit()

    def selectMarked(self):
    	if not pybel:
    		return
        if not self.showFragments:
            molecules=[i.context.molecule for i in self.imageWidgets]
            fMap=map_fragments([self.selectedFragment], molecules)
            for i in self.imageWidgets:
                if fMap[i.context.molecule][self.selectedFragment]:
                    i.selected=True
                else:
                    i.selected=False
                i.repaint()
        if self.commitOnChange:
            self.commit()
    
    def commit(self):
        if self.showFragments:
            sAttr=self.candidateMolSmilesAttr[self.moleculeSmilesAttr]
            molecules=[str(e[sAttr]) for e in self.molData]
            fragments=[i.context.molecule for i in self.imageWidgets if i.selected]
            fragmap=map_fragments(fragments, molecules)
            match=filter(lambda m:max(fragmap[m].values()), molecules)
            examples=[e for e in self.molData if str(e[sAttr]) in match]
            if examples:
                table=orange.ExampleTable(examples)
                self.send("Selected Molecules", table)
            else:
                self.send("Selected Molecules", None)                
        else:
            mols=[i.context.molecule for i in self.imageWidgets if i.selected]
            sAttr=self.candidateMolSmilesAttr[self.moleculeSmilesAttr]
            examples=[e for e in self.molData if str(e[sAttr]) in mols]
            if examples:
                table=orange.ExampleTable(examples)
                self.send("Selected Molecules", table)
            else:
                self.send("Selected Molecules", None)

    def keyPressEvent(self, key):
        if key.key()==Qt.Key_Control:
            self.ctrlPressed=TRUE
        else:
            OWWidget.keyPressEvent(self, key)

    def keyReleaseEvent(self, key):
        if key.key()==Qt.Key_Control:
            self.ctrlPressed=FALSE
        else:
            OWWidget.keyReleaseEvent(self, key)

    def saveToHTML(self):
        fileName=str(QFileDialog.getSaveFileName("index.html","HTML (.html)", None, "Save to.."))
        if not fileName:
            return
        else:
            file=open(fileName, "w")
        import os
        path, _ =os.path.split(fileName)
        if "molimages" not in os.listdir(path):
            os.mkdir(path+"/molimages")
        title="Molekule"
        file.write("<html><title>"+title+"</title>\n")
        file.write("<body> <table border=\"1\">\n")
        i=0
        try:
            import Image
        except:
            pass
        for row in range(len(self.imageWidgets)/self.numColumns+1):
            file.write("<tr>\n")
            for col in range(self.numColumns):
                if i>=len(self.imageWidgets):
                    break
                try:
                    im=Image.open(self.imageWidgets[i].context.imagename)
                    if im.mode!="RGB":
                        im=im.convert("RGB")
                    im.save(path+"/molimages/image"+str(i)+".gif", "GIF")
                    file.write("<td><p>"+str(self.imageWidgets[i].label.text())+"</p><img src=\"./molimages/image"+str(i)+".gif\"></td>\n")
                except:
                    from shutil import copy
                    copy(self.imageWidgets[i].context.imagename, path+"/molimages/")
                    file.write("<td><p>"+str(self.imageWidgets[i].label.text())+"</p><img src=\"./molimages/image"+str(i)+".bmp\"></td>\n")
                i+=1
            file.write("</tr>\n")
        file.write("</table></body></html>")
        file.close()

    def saveSettings(self, *args, **kw):
        OWWidget.saveSettings(self, *args, **kw)
        ImageCache().sync()

def molecule2BMP(molSmiles, filename, size=200, title="", grayedBackground=False):
    moleculeFragment2BMP(molSmiles, None, filename)

def moleculeFragment2BMP(molSmiles, fragSmiles, filename, size=200, title="", grayedBackground=False):
    obiChem.mol_to_svg(molSmiles, fragSmiles, filename)

def remoteMoleculeFragment2BMP(molSmiles, fragSmiles, filename, size=200, title="", grayedBackground=False):
    import cStringIO
    import urllib
    import Image, ImageDraw

    from openbabel import OBConversion, OBMol
    loader = OBConversion()
    loader.SetInAndOutFormats("smi", "smi")
    if not loader.ReadString(OBMol(), molSmiles):
        img = Image.new("RGB", (size, size), (255,255,255))
        draw = ImageDraw.Draw(img)
        draw.text((10, size/2-10), "wrong SMILES notation", fill=(0, 0, 0))
        img.save(filename)
        return
    
    params = urllib.urlencode({'password': OWMoleculeVisualizer.serverPwd, 'molSmiles': molSmiles, 'fragSmiles': fragSmiles, 'size': size, 'title': title, 'grayBack': grayedBackground})
    try:
        f = urllib.urlopen("http://212.235.189.53/openEye/drawMol.py", params)
        imgstr = f.read()
    except IOError, er:
        #print er
        raise er
    im = cStringIO.StringIO(imgstr)
    img = Image.open(im)
    #print img.format, img.size, img.mode
    img.save(filename)
    #del img

def remoteMolecule2BMP(molSmiles, filename, size=200, title="", grayedBackground=False):
    import cStringIO
    import urllib
    import Image, ImageDraw

    from openbabel import OBConversion, OBMol
    loader = OBConversion()
    loader.SetInAndOutFormats("smi", "smi")
    if not loader.ReadString(OBMol(), molSmiles):
        img = Image.new("RGB", (size, size), (255,255,255))
        draw = ImageDraw.Draw(img)
        draw.text((10, size/2-10), "wrong SMILES notation", fill=(0, 0, 0))
        img.save(filename)
        return    
    
    params = urllib.urlencode({'password': OWMoleculeVisualizer.serverPwd, 'molSmiles': molSmiles, 'size': size, 'title': title, 'grayBack': grayedBackground})
    try:
        f = urllib.urlopen("http://212.235.189.53/openEye/drawMol.py", params)
        imgstr = f.read()
    except IOError, er:
        #print er, "rm2BMP"
        raise er
    im = cStringIO.StringIO(imgstr)
    img = Image.open(im)
    #print img.format, img.size, img.mode
    img.save(filename)
    #del img        


def map_fragments(fragments, smiles, binary=True):
    ret = {}
    for s in smiles:
        mol = pybel.readstring("smi", s)
        d={}
        for f in fragments:
            pat = pybel.Smarts(f)
            count=0
            if pat.findall(mol):
                count+=1
            if binary:
                d[f]=count!=0 and 1 or 0
            else:
                d[f]=count
        ret[s]=d
    return ret

if not oasaLocal:
    moleculeFragment2BMP = remoteMoleculeFragment2BMP
    molecule2BMP = remoteMolecule2BMP

if __name__=="__main__":
    pass
##    app=QApplication(sys.argv)
##    from pywin.debugger import set_trace
####    set_trace()
##    w=OWMoleculeVisualizer()
####    app.setMainWidget(w)
##    w.show()
##    data=orange.ExampleTable("E://chem/chemdata/BCMData_growth_frag.tab")
##    
##    w.setMoleculeTable(data)
####    data=orange.ExampleTable("E://chem//new//sf.tab")
####    w.setFragmentTable(data)
##    app.exec_()
##    w.saveSettings()



"""
========================================
Gene Expression Atlas (``obiGeneAtlas``)
========================================

Interface to Gene Expression Atlas.

`Gene Expression Atlas <http://www.ebi.ac.uk/gxa/>`_ is a curated subset of
gene expression experiments in Array Express Archive.

.. autofunction:: gene_expression_atlas

.. autofunction:: default_gene_matcher

.. autofunction:: to_taxid

"""

from __future__ import absolute_import

import os, shelve, sys
from collections import defaultdict, namedtuple
from contextlib import closing

from Orange.orng import orngServerFiles
from Orange.utils import serverfiles

from . import obiGene

GeneResults = namedtuple("GeneResults", "id name synonyms expressions")
ExpressionResults = namedtuple("ExpressionResults", "ef efv up down experiments")
ExperimentExpression = namedtuple("ExperimentExpression", "accession expression pvalue")

##
GeneAtlasResult = GeneResults
AtlasExpressions = ExpressionResults
AtlasExperiment = ExperimentExpression
##

CACHE_VERSION = 1


def _cache(name="AtlasGeneResult.shelve"):
    """ Return a open cache instance (a shelve object).
    """
    if not os.path.exists(orngServerFiles.localpath("GeneAtlas")):
        try:
            os.makedirs(orngServerFiles.localpath("GeneAtlas"))
        except OSError:
            pass
    cache = shelve.open(orngServerFiles.localpath("GeneAtlas", name))
    if cache.get(name + "__CACHE_VERSION__", None) == CACHE_VERSION:
        return cache
    else:
        cache = shelve.open(orngServerFiles.localpath("GeneAtlas", name), "n")
        cache[name + "__CACHE_VERSION__"] = CACHE_VERSION
        return cache


SLEEP_TIME_MULTIPLIER = 3.0

def gene_expression_atlas(genes, progress_callback=None):
    """ Return GeneResults instances for genes (genes must be valid ensembl ids).
    """
    import time
    genes = list(genes)
    result_dict = {}
    genes_not_cached = []
    # See which genes are already cached
    with closing(_cache()) as cache:
        for gene in genes:
            if str(gene) in cache:
                result_dict[gene] = cache[str(gene)]
            else:
                genes_not_cached.append(gene)
    
    batch_size = 10
    start = 0
    res = []
    while start < len(genes_not_cached):
        batch = genes_not_cached[start: start + batch_size]
        start += batch_size
        start_time = time.time()
        batch_res = batch_gene_atlas_expression(batch)
        # Cache the new results.
        # TODO: handle genes without any results.
        genes_with_no_results = set(batch) - set(r.id for r in batch_res) 
        with closing(_cache()) as cache:
            for atlas_res in batch_res:
                cache[str(atlas_res.id)] = atlas_res
                result_dict[atlas_res.id] = atlas_res
            for g in genes_with_no_results:
                cache[str(g)] = None
        res.extend(batch_res)
        # Sleep
        if start % (batch_size * 10) == 0:
            # every 10 batches wait one minute before continuing. 
            time.sleep(60)
        else:
            time.sleep(min(20.0, SLEEP_TIME_MULTIPLIER*(time.time() - start_time)))
            
        if progress_callback:
            progress_callback(100.0 * start / len(genes_not_cached))
    
    return [result_dict.get(g, None) for g in genes]

    
def batch_gene_atlas_expression(genes):
    cond = GenePropertyCondition("Ensgene", "Is", genes)
    res = run_query(cond, format="json")
    results = res["results"]
    results_genes = []
    for one_result in results:
        gene = one_result["gene"]
        id = gene["id"]
        name = gene["name"]
        synonyms = gene.get("synonyms", [])
        expressions = one_result["expressions"]
        result_expressions = []
        for expression in expressions:
            ef = expression["ef"]
            efv = expression["efv"]
            up = expression["upExperiments"]
            down = expression["downExperiments"]
            experiments = expression["experiments"]
            result_experiment = []
            for exp in experiments:
                if "accession" in exp:
                    exp_accession = exp["accession"]
                elif "experimentAccession" in exp:
                    exp_accession = exp["experimentAccession"]
                else:
                    raise KeyError()
                if "expression" in exp:
                    updown = exp["expression"]
                elif "updn" in exp:
                    updown = exp["updn"]
                else:
                    raise KeyError
                pval = exp["pvalue"]
                result_experiment.append(ExperimentExpression(exp_accession, updown, pval))
            result_expressions.append(ExpressionResults(ef, efv, up, down, result_experiment))
        results_genes.append(GeneResults(id, name, synonyms, result_expressions))
    return results_genes


def default_gene_matcher(organism):
    """ Return a default gene matcher for organism
    (targeting Ensembl gene ids).
    
    """
    taxid = to_taxid(organism)
    matcher = obiGene.matcher([obiGene.GMEnsembl(taxid),
                               obiGene.GMNCBI(taxid)])
    matcher.set_targets(obiGene.EnsembleGeneInfo(taxid).keys())
    return matcher

from Orange.utils import lru_cache

@lru_cache(maxsize=3)
def _cached_default_gene_matcher(organism): 
    return default_gene_matcher(organism)
    

def get_atlas_summary(genes, organism, gene_matcher=None,
                      progress_callback=None):
    """ Return 3 dictionaries containing a summary of atlas information
    about three experimental factors:
    
        - Organism Part (OP) 
        - Disease State (DS)
        - Cell type (CT)
    
    Each dictionary contains query genes as keys. Values are dictionaries
    mapping factor values to a 2-tuple containig the count of up regulated
    and down regulated experiments.
    
    Example ::
    
        >>> get_atlas_summary(["RUNX1"], "Homo sapiens")
        ({u'RUNX1': ...
        
    """
    if gene_matcher is None:
        gene_matcher = _cached_default_gene_matcher(organism)
        
    matched, unmatched = [], []
    for gene, match in zip(genes, map(gene_matcher.umatch, genes)):
        if match:
            matched.append(match)
        else:
            unmatched.append(gene)
    if unmatched:
        import warnings
        warnings.warn("Unmatched genes " + "," .join(["%r" % g for g in unmatched]))
    
    results = gene_expression_atlas(matched, progress_callback=progress_callback)
    
    def collect_ef_summary(result, ef, summary):
        for exp in result.expressions:
            if exp.ef == ef:
                if any([exp.up, exp.down]):
                    summary[result.name][exp.efv] = (exp.up, exp.down)
        
            
    op, ds, ct = defaultdict(dict), defaultdict(dict), defaultdict(dict)
    for res in results:
        if res:
            collect_ef_summary(res, "organism_part", op)
            collect_ef_summary(res, "disease_state", ds)
            collect_ef_summary(res, "cell_type", ct)
        
    return dict(op), dict(ds), dict(ct)

def drop_none(iter):
    """ Drop all ``None`` from the iterator. 
    """
    for e in iter:
        if e is not None:
            yield e
            
def construct_atlas_gene_sets(genes, organism, factors=["organism_part",
                                    "disease_state", "cell_type"],
                              max_pvalue=1e-5):
    """ Construct gene sets for atlas experimental factor values in
    ``factors``.
    """
    results = gene_expression_atlas(genes)
    sets = defaultdict(list)
    
    for res in drop_none(results):
        for exp in res.expressions:
            if exp.ef not in factors:
                continue
            diff_exp = [e for e in exp.experiments \
                        if e.pvalue <= max_pvalue]
            if diff_exp:
                sets[exp.ef, exp.efv].append(res.id)

    organism = "+".join(organism.lower().split())
    from .obiGeneSets import GeneSets, GeneSet
    
    def display_string(name):
        return name.capitalize().replace("_", " ")
    
    gene_sets = []
    for (ef, efv), genes in sets.items():
        ef_display = display_string(ef)
        gs = GeneSet(genes, "Diff. expressed in %s=%s." % (ef_display, efv), id=ef + ":" + efv,
                     description="Diff. expressed in %s=%s" % (ef_display, efv),
                     link="http://www.ebi.ac.uk/gxa/qrs?specie_0={organism}&gprop_0=&gnot_0=&gval_0=&fact_1=&fexp_1=UPDOWN&fmex_1=&fval_1=%22{efv}%22+&view=hm".format( \
                            organism=organism, efv="+".join(efv.lower().split())),
                     hierarchy=("Tissues", ef_display))
        gene_sets.append(gs)
    return GeneSets(gene_sets)


# Mapping for common taxids from obiTaxonomy
TAXID_TO_ORG = {"": "Anopheles gambiae",
                "3702": "Arabidopsis thaliana",
                "9913": "Bos taurus",
                "6239": "Caenorhabditis elegans",
                "7955": "Danio rerio",
                "7227": "Drosophila melanogaster",
                "": "Epstein barr virus",
                "": "Gallus gallus",
                "9606": "Homo sapiens",
                "": "Human cytomegalovirus",
                "": "Kaposi sarcoma-associated herpesvirus",
                "10090": "Mus musculus",
                "10116": "Rattus norvegicus",
                "4932": "Saccharomyces cerevisiae",
                "4896": "Schizosaccharomyces pombe",
                "8355": "Xenopus laevis"
     }

def to_taxid(name):
    dd = dict((v, k) for k, v in TAXID_TO_ORG.items())
    if name in dd:
        return dd[name]
    else:
        from . import obiTaxonomy as tax
        ids = tax.to_taxid(name, mapTo=TAXID_TO_ORG.keys())
        if ids:
            return ids.pop()
        else:
            raise ValueError("Unknown organism.")


__doc__ += """\
Low level REST query interface
------------------------------

Use `query_atlas_simple` for simple querys.

Example (query human genes for experiments in which they are up regulated) ::

    >>> run_simple_query(genes=["SORL1", "PSIP1", "CDKN1C"], regulation="up", organism="Homo sapiens")
    {u'...
    
Or use the `AtlasCondition` subclasses in this module to construct a more
advanced query and use the `run_query` function.

Example (query human genes annotated to the GO term 'transporter activity'
that are up regulated in the liver in at least three experiments) ::

    >>> go_cond = GenePropertyCondition("Goterm", "Is", "transporter activity")
    >>> liver_cond = ExperimentalFactorCondition("Organism_part", "up", 3, "liver")
    >>> org_cond = OrganismCondition("Homo sapiens")
    >>> cond_list = ConditionList([go_cond, liver_cond, org_cond])
    >>> run_query(cond_list)
    {u'...

"""

import urllib2
from  StringIO import StringIO
import json
from xml.etree.ElementTree import ElementTree

parse_json = json.load


def parse_xml(stream):
    """ Parse an xml stream into an instance of xml.etree.ElementTree.ElementTree.
    """
    return ElementTree(file=stream)


class GeneExpressionAtlasConenction(object):
    """ A connection to Gene Expression Atlas database.
    """
    DEFAULT_ADDRESS = "http://www.ebi.ac.uk:80/gxa/"
    DEFAULT_CACHE = orngServerFiles.localpath("GeneAtlas", "GeneAtlasConnectionCache.shelve")
    def __init__(self, address=None, timeout=30, cache=None):
        """ Initialize the conenction.
        
        :param address: Address of the server.
        :param timeout: Socket timeout.
        :param cache : A dict like object to use as a cache.
        
        """
        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        self.cache = cache if cache is not None else self.DEFAULT_CACHE
    
    def query(self, condition, format="json", start=None, rows=None, indent=False):
        url = self.address + "api/vx?" + condition.rest()
        if start is not None and rows is not None:
            url += "&start={0}&rows={1}".format(start, rows)
        url += "&format={0}".format(format)
        if indent:
            url += "&indent"
#        print url
        if self.cache is not None:
            return self._query_cached(url, format)
        else:
            return urllib2.urlopen(url)
        return response
    
    def _query_cached(self, url, format):
        if self.cache is not None:
            with self.open_cache() as cache:
                cached = url in cache
            
            if not cached:
                response = urllib2.urlopen(url)
                contents = response.read()
                # Test if the contents is a valid json or xml string (some 
                # times the stream just stops in the middle, so this makes
                # sure we don't cache an invalid response
                # TODO: what about errors (e.g. 'cannot handle the
                # query in a timely fashion'
                if format == "json":
                    parse_json(StringIO(contents))
                else:
                    parse_xml(StringIO(contents))
                    
                with self.open_cache() as cache:
                    cache[url] = contents
            else:
                with self.open_cache() as cache:
                    contents = cache[url]
            return StringIO(contents)
        else:
            return urllib2.urlopen(url)
        
    def open_cache(self):
        if isinstance(self.cache, basestring):
            return closing(shelve.open(self.cache))
        elif hasattr(self.cache, "close"):
            return closing(self.cache)
        elif self.cache is None:
            return fake_closing({})
        else:
            return fake_closing(self.cache)
        
        
from contextlib import contextmanager
@contextmanager
def fake_closing(obj):
    yield obj
    
    
# Names of all Gene Property filter names
GENE_FILTERS = \
    ["Name", # Gene name
     "Goterm", #Gene Ontology Term
     "Interproterm", #InterPro Term
     "Disease", #Gene-Disease Assocation
     "Keyword", #Gene Keyword
     "Protein", #Protein

     "Dbxref", #Other Database Cross-Refs
     "Embl", #EMBL-Bank ID
     "Ensfamily", #Ensembl Family
     "Ensgene", #Ensembl Gene ID

     "Ensprotein", #Ensembl Protein ID
     "Enstranscript", #Ensembl Transcript ID
     "Goid", #Gene Ontology ID
     "Image", #IMAGE ID
     "Interproid", #InterPro ID
     "Locuslink", #Entrez Gene ID

     "Omimid", #OMIM ID
     "Orf", #ORF
     "Refseq", #RefSeq ID
     "Unigene", #UniGene ID
     "Uniprot", #UniProt Accession

     "Hmdb", #HMDB ID
     "Chebi", #ChEBI ID
     "Cas", #CAS
     "Uniprotmetenz", #Uniprotmetenz
     "Gene", #Gene Name or Identifier
     "Synonym", #Gene Synonym
     ]
    
# Valid Gene Property filter qualifiers 
GENE_FILTER_QUALIFIERS =\
    ["Is",
     "IsNot"
     ]

# Organisms in the Atlas
ATLAS_ORGANISMS = \
    ["Anopheles gambiae",
     "Arabidopsis thaliana",
     "Bos taurus",
     "Caenorhabditis elegans",
     "Danio rerio",
     "Drosophila melanogaster",
     "Epstein barr virus",
     "Gallus gallus",
     "Homo sapiens",
     "Human cytomegalovirus",
     "Kaposi sarcoma-associated herpesvirus",
     "Mus musculus",
     "Rattus norvegicus",
     "Saccharomyces cerevisiae",
     "Schizosaccharomyces pombe",
#     "Unknown",
     "Xenopus laevis"
     ]
    
def ef_ontology():
    """ Return the `EF <http://www.ebi.ac.uk/efo/>`_ (Experimental Factor) ontology
    """
    from . import obiOntology
    from . import orngServerFiles
    # Should this be in the OBOFoundry (Ontology) domain
    try:
        file = open(orngServerFiles.localpath_download("ArrayExpress", "efo.obo"), "rb")
    except urllib2.HTTPError:
        file = urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo")
    return obiOntology.OBOOntology(file)


class Condition(object):
    """ Base class for Gene Expression Atlas query condition
    """
    def validate(self):
        """ Validate condition in a subclass.
        """
        raise NotImplementedError
    
    def rest(self):
        """ Return a REST query part in a subclass.
        """
        raise NotImplementedError
    
    
class ConditionList(list, Condition):
    """ A list of AtlasCondition instances.
    """ 
    def validate(self):
        for item in self:
            item.validate()
        
    def rest(self):
        return "&".join(cond.rest() for cond in self)


class GenePropertyCondition(Condition):
    """ An atlas gene filter condition.
    
    :param property: Property of the gene. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Is' or 'IsNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a gene name
        >>> condition = GenePropertyCondition("Name", "Is", "AS3MT")
        >>> # Condition on genes from a GO Term
        >>> condition = GenePropertyCondition("Goterm", "Is", "p53 binding")
        >>> # Condition on disease association
        >>> condition = GenePropertyCondition("Disease", "Is", "cancer")
        
    """
    def __init__(self, property, qualifier, value):
        self.property = property or ""
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        assert(self.property in GENE_FILTERS + [""])
        assert(self.qualifier in GENE_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "gene{property}{qualifier}={value}".format(**self.__dict__)
        
        
class ExperimentalFactorCondition(Condition):
    """ An atlas experimental factor filter condition.
    
    :param factor: EFO experiamntal factor
    :param regulation: "up", "down", "updown", "any" or "none"
    :param n: Minimum number of of experimants with this condition
    :param value: Experimantal factor value
    
    Example ::
    
        >>> # Any genes up regulated in at least 3 experiments involving cancer.
        >>> condition = ExperimentalFactorCondition("", "up", 3, "cancer")
        >>> # Only genes which are up/down regulated in the heart in at least one experiment. 
        >>> condition = ExperimentalFactorCondition("Organism_part", "updown", 1, "heart")
        
    """
    def __init__(self, factor, regulation, n, value):
        self.factor = factor
        self.regulation = regulation
        self.n = n
        self.value = value
        self.validate()
        
    def validate(self):
        # TODO: validate the factor and value
#        assert(self.factor in ef_ontology())
        assert(self.regulation in ["up", "down", "updown"])
        
    def rest(self):
        return "{regulation}{n}In{factor}={value}".format(**self.__dict__)
        
        
class OrganismCondition(Condition):
    """ Condition on organism.
    """
    def __init__(self, organism):
        self.organism = organism
        self.validate()
        
    def validate(self):
        assert(self.organism in ATLAS_ORGANISMS)
        
    def rest(self):
        return "species={0}".format(self.organism.replace(" ", "+").lower())
        
        
class ExperimentCondition(Condition):
    """ Condition on experiement
    
    :param property: Property of the experiment. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Has' or 'HasNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a experiemnt acession
        >>> condition = ExperimentCondition("", "", "E-GEOD-24283")
        >>> # Condition on experiments involving lung
        >>> condition = ExperimentCondition("Organism_part", "Has", "lung")
        
    """
    EXPERIMENT_FILTER_QUALIFIERS = [
                "Has",
                "HasNot"]
    
    def __init__(self, property, qualifier, value):
        self.property = property
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        # TODO: check to EFO factors
#        assert(self.property in EXPERIMENT_FILTERS + [""])
        assert(self.qualifier in self.EXPERIMENT_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "experiment{property}{qualifier}={value}".format(**self.__dict__)
        
        
class GeneExpressionAtlasError(Exception):
    """ An error response from the Atlas server.
    """
    pass
    
    
def __check_atlas_error_json(response):
    if "error" in response:
        raise GeneExpressionAtlasError(response["error"])
    return response
 
     
def __check_atlas_error_xml(response):
    error = response.find("error")
    if error is not None:
        raise GeneExpressionAtlasError(error.text)
    return response
    
        
def run_simple_query(genes=None, regulation=None, organism=None,
                     condition=None, format="json", start=None,
                     rows=None):
    """ A simple Gene Atlas query.
    
    :param genes: A list of gene names to search for.
    :param regulation: Search for experiments in which `genes` are "up",
        "down", "updown" or "none" regulated. If None all experiments
        are searched.
    :param organism: Search experiments for organism. If None all experiments
        are searched.
    :param condition: An EFO factor value (e.g. "brain")
    
    Example ::
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], organism="Mus musculus")
        {u'...
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], regulation="up", organism="Mus musculus")
        {u'...
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], regulation="up", condition="liver", organism="Mus musculus")
        {u'...
        
    """
    conditions = ConditionList()
    if genes:
        conditions.append(GenePropertyCondition("Gene", "Is", genes))
    if regulation or condition:
        regulation = "any" if regulation is None else regulation
        condition = "" if condition is None else condition
        conditions.append(ExperimentalFactorCondition("", regulation, 1, condition))
    if organism:
        conditions.append(OrganismCondition(organism))
        
    connection = GeneExpressionAtlasConenction()
    results = connection.query(conditions, format=format, start=start,
                               rows=rows)
    if format == "json":
        return parse_json(results)
    else:
        return parse_xml(results)

"""\
.. todo:: can this be implemented query_atlas(organism="...", Locuslink="...", Chebi="...", up3InCompound="..." downInEFO="...")
      Need a full list of accepted factors 
"""

def run_query(condition, format="json", start=None, rows=None, indent=False, connection=None):
    """ Query Atlas based on a `condition` (instance of :class:`Condition`)
    
    Example ::
        
        >>> condition1 = GenePropertyCondition("Goterm", "Is", "p53 binding")
        >>> condition2 = ExperimentalFactorCondition("Organism_part", "up", 3, "heart")
        >>> condition = ConditionList([condition1, condition2])
        >>> run_query(condition)
        {u'...
        
    """
    if connection is None:
        connection = GeneExpressionAtlasConenction()
    results = connection.query(condition, format=format, start=start,
                               rows=rows, indent=indent)
    if format == "json":
        response = parse_json(results)
        return __check_atlas_error_json(response)
    else:
        response = parse_xml(results)
        return __check_atlas_error_xml(response)
    
def test():
    from pprint import pprint    
    pprint(get_atlas_summary(['Pou5f1', 'Dppa3'], 'Mus musculus'))
       
    pprint(get_atlas_summary(['PDLIM5', 'FGFR2' ], 'Homo sapiens'))
    import doctest 
    doctest.testmod(optionflags=doctest.ELLIPSIS)
    
if __name__ == "__main__":
    test()



"""<name>GEO Data Sets</name>
<description>Access to Gene Expression Omnibus data sets.</description>
<priority>20</priority>
<contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si)</contact>
<icon>icons/GEODataSets.png</icon>
"""

from __future__ import absolute_import, with_statement

from collections import defaultdict
from functools import partial 
import sys, os, glob

from Orange.orng import orngServerFiles
from Orange.orng.orngDataCaching import data_hints
from Orange.OrangeWidgets import OWGUI, OWGUIEx
from Orange.OrangeWidgets.OWWidget import *

from .. import obiGEO

LOCAL_GDS_COLOR = Qt.darkGreen

TextFilterRole = OWGUI.OrangeUserRole.next()

class TreeModel(QAbstractItemModel):
    def __init__(self, data, header, parent):
        QAbstractItemModel.__init__(self, parent)
        self._data = [[QVariant(s) for s in row] for row in data]
        self._dataDict = {}
        self._header = {Qt.Horizontal: dict([(i, {Qt.DisplayRole: h}) for i, h in enumerate(header)])}
        self._roleData = {Qt.DisplayRole:self._data}
        dataStore = partial(defaultdict, partial(defaultdict, partial(defaultdict, QVariant)))
        self._roleData = dataStore(self._roleData)
        self._header = dataStore(self._header)
    
    def setColumnLinks(self, column, links):
        font =QFont()
        font.setUnderline(True)
        font = QVariant(font)
        for i, link in enumerate(links):
            self._roleData[LinkRole][i][column] = QVariant(link)
            self._roleData[Qt.FontRole][i][column] = font
            self._roleData[Qt.ForegroundRole][i][column] = QVariant(QColor(Qt.blue))
    
    def setRoleData(self, role, row, col, data):
        self._roleData[role][row][col] = data
        
    def setData(self, index, value, role=Qt.EditRole):
        self._roleData[role][index.row()][index.column()] = value
        self.emit(SIGNAL("dataChanged(QModelIndex, QModelIndex)"), index, index)
        
    def data(self, index, role):
        row, col = index.row(), index.column()
        return self._roleData[role][row][col]
        
    def index(self, row, col, parent=QModelIndex()):
        return self.createIndex(row, col, 0)
    
    def parent(self, index):
        return QModelIndex()
    
    def rowCount(self, index=QModelIndex()):
        if index.isValid():
            return 0
        else:
            return len(self._data)
        
    def columnCount(self, index):
        return len(self._header[Qt.Horizontal])

    def headerData(self, section, orientation, role):
        try:
            return QVariant(self._header[orientation][section][role])
        except KeyError, er:
#            print >> sys.stderr, er
            return QVariant()
        
    def setHeaderData(self, section, orientation, value, role=Qt.EditRole):
        self._header[orientation][section][role] = value
        
from Orange.utils import lru_cache

class MySortFilterProxyModel(QSortFilterProxyModel):    
    def __init__(self, parent=None):
        QSortFilterProxyModel.__init__(self, parent)
        self._filter_strings = []
        self._cache = {}
        self._cache_fixed = {}
        self._cache_prefix = {}
        self._row_text = {}
        
        # Create a cached version of _filteredRows
        self._filteredRows = lru_cache(100)(self._filteredRows) 

    def setSourceModel(self, model):
        """ Set the source model for the filter
        """ 
        self._filter_strings = []
        self._cache = {}
        self._cache_fixed = {}
        self._cache_prefix = {}
        self._row_text = {}
        QSortFilterProxyModel.setSourceModel(self, model)
        
    def addFilterFixedString(self, string, invalidate=True):
        """ Add `string` filter to the list of filters. If invalidate is
        True the filter cache will be recomputed.
        """
        self._filter_strings.append(string)
        all_rows = range(self.sourceModel().rowCount())
        row_text = [self.rowFilterText(row) for row in all_rows]
        self._cache[string] = [string in text for text in row_text]
        if invalidate:
            self.updateCached()
            self.invalidateFilter()
        
    def removeFilterFixedString(self, index=-1, invalidate=True):
        """ Remove the `index`-th filter string. If invalidate is True the
        filter cache will be recomputed.
        """
        string = self._filter_strings.pop(index) 
        del self._cache[string] 
        if invalidate:
            self.updateCached()
            self.invalidateFilter()
            
    def setFilterFixedStrings(self, strings):
        """ Set a list of string to be the new filters.
        """
        s_time = time.time()
        to_remove = set(self._filter_strings) - set(strings)
        to_add = set(strings) - set(self._filter_strings)
        for str in to_remove:
            self.removeFilterFixedString(self._filter_strings.index(str), invalidate=False)
        
        for str in to_add:
            self.addFilterFixedString(str, invalidate=False)
        self.updateCached()
        self.invalidateFilter()
            
    def _filteredRows(self, filter_strings):
        """ Return a dictionary mapping row indexes to True False values.
        .. note:: This helper function is wrapped in the __init__ method. 
        """
        all_rows = range(self.sourceModel().rowCount())
        cache = self._cache
        return dict([(row, all([cache[str][row] for str in filter_strings])) for row in all_rows])
    
    def updateCached(self):
        """ Update the combined filter cache.
        """
        self._cache_fixed = self._filteredRows(tuple(sorted(self._filter_strings))) 
        
    def setFilterFixedString(self, string):
        """Should this raise an error? It is not being used.
        """
        QSortFilterProxyModel.setFilterFixedString(self, string)
        
    def rowFilterText(self, row):
        """ Return text for `row` to filter on. 
        """
        f_role = self.filterRole()
        f_column = self.filterKeyColumn()
        s_model = self.sourceModel()
        data = s_model.data(s_model.index(row, f_column), f_role)
        if isinstance(data, QVariant):
            data = unicode(data.toString(), errors="ignore")
        else:
            data = unicode(data, errors="ignore")
        return data
        
    def filterAcceptsRow(self, row, parent): 
        return self._cache_fixed.get(row, True)
    
    def lessThan(self, left, right):
        if left.column() == 1 and right.column(): # TODO: Remove fixed column handling
            left_gds = str(left.data(Qt.DisplayRole).toString())
            right_gds = str(right.data(Qt.DisplayRole).toString())
            left_gds = left_gds.lstrip("GDS")
            right_gds = right_gds.lstrip("GDS")
            try:
                return int(left_gds) < int(right_gds)
            except Exception, ex:
                pass
        return QSortFilterProxyModel.lessThan(self, left, right)
    
from Orange.OrangeWidgets.OWGUI import LinkStyledItemDelegate, LinkRole

def childiter(item):
    """ Iterate over the children of an QTreeWidgetItem instance.
    """
    for i in range(item.childCount()):
        yield item.child(i)
                
class OWGEODatasets(OWWidget):
    settingsList = ["outputRows", "mergeSpots", "gdsSelectionStates", "splitterSettings", "currentGds", "autoCommit"]

    def __init__(self, parent=None ,signalManager=None, name=" GEO Data Sets"):
        OWWidget.__init__(self, parent ,signalManager, name)

        self.outputs = [("Expression Data", ExampleTable)]

        ## Settings
#        self.selectedSubsets = []
#        self.sampleSubsets = []
        self.selectedAnnotation = 0
        self.includeIf = False
        self.minSamples = 3
        self.autoCommit = False
        self.outputRows = 0
        self.mergeSpots = True
        self.filterString = ""
        self.currentGds = None
        self.selectionChanged = False
        self.autoCommit = False
        self.gdsSelectionStates = {}
        self.splitterSettings = ['\x00\x00\x00\xff\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x01\xea\x00\x00\x00\xd7\x01\x00\x00\x00\x07\x01\x00\x00\x00\x02',
                                 '\x00\x00\x00\xff\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x01\xb5\x00\x00\x02\x10\x01\x00\x00\x00\x07\x01\x00\x00\x00\x01']

        self.loadSettings()

        ## GUI
        self.infoBox = OWGUI.widgetLabel(OWGUI.widgetBox(self.controlArea, "Info", addSpace=True), "\n\n")

        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace=True)
        OWGUI.radioButtonsInBox(box, self, "outputRows", ["Genes or spots", "Samples"], "Rows", callback=self.commitIf)
        OWGUI.checkBox(box, self, "mergeSpots", "Merge spots of same gene", callback=self.commitIf)

        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace=True)
        self.commitButton = OWGUI.button(box, self, "Commit", callback=self.commit)
        cb = OWGUI.checkBox(box, self, "autoCommit", "Commit on any change")
        OWGUI.setStopper(self, self.commitButton, cb, "selectionChanged", self.commit)
        OWGUI.rubber(self.controlArea)

        self.filterLineEdit = OWGUIEx.lineEditHint(self.mainArea, self, "filterString", "Filter",
                                   caseSensitive=False, matchAnywhere=True, 
                                   #listUpdateCallback=self.filter, callbackOnType=False, 
                                   callback=self.filter,  delimiters=" ")
        
        splitter = QSplitter(Qt.Vertical, self.mainArea)
        self.mainArea.layout().addWidget(splitter)
        self.treeWidget = QTreeView(splitter)
        
        self.treeWidget.setSelectionMode(QAbstractItemView.SingleSelection)
        self.treeWidget.setRootIsDecorated(False)
        self.treeWidget.setSortingEnabled(True)
        self.treeWidget.setAlternatingRowColors(True)
        self.treeWidget.setUniformRowHeights(True)
        self.treeWidget.setItemDelegate(LinkStyledItemDelegate(self.treeWidget))
        self.treeWidget.setItemDelegateForColumn(0, OWGUI.IndicatorItemDelegate(self.treeWidget, role=Qt.DisplayRole))
        
        self.connect(self.treeWidget, SIGNAL("itemSelectionChanged ()"), self.updateSelection)
        self.treeWidget.viewport().setMouseTracking(True)
        
        splitterH = QSplitter(Qt.Horizontal, splitter) 
        
        box = OWGUI.widgetBox(splitterH, "Description")
        self.infoGDS = OWGUI.widgetLabel(box, "")
        self.infoGDS.setWordWrap(True)
        OWGUI.rubber(box)
        
        box = OWGUI.widgetBox(splitterH, "Sample Annotations")
        self.annotationsTree = QTreeWidget(box)
        self.annotationsTree.setHeaderLabels(["Type (Sample annotations)", "Sample count"])
        self.annotationsTree.setRootIsDecorated(True)
        box.layout().addWidget(self.annotationsTree)
        self.connect(self.annotationsTree, SIGNAL("itemChanged(QTreeWidgetItem * , int)"), self.annotationSelectionChanged)
        self._annotationsUpdating = False
        self.splitters = splitter, splitterH
        self.connect(splitter, SIGNAL("splitterMoved(int, int)"), self.splitterMoved)
        self.connect(splitterH, SIGNAL("splitterMoved(int, int)"), self.splitterMoved)
        
        for sp, setting in zip(self.splitters, self.splitterSettings):
            sp.restoreState(setting)
            
        self.searchKeys = ["dataset_id", "title", "platform_organism", "description"]
        self.cells = []
        
        QTimer.singleShot(50, self.updateTable)
        self.resize(1000, 600)

    def updateInfo(self):
        gds_info = self.gds_info #obiGEO.GDSInfo()
        text = "%i datasets\n%i datasets cached\n" % (len(gds_info), len(glob.glob(orngServerFiles.localpath("GEO") + "/GDS*")))
        filtered = self.treeWidget.model().rowCount()
        if len(self.cells) != filtered:
            text += ("%i after filtering") % filtered
        self.infoBox.setText(text)
        
    def updateTable(self):
        self.treeItems = []
        self.progressBarInit()
        with orngServerFiles.DownloadProgress.setredirect(self.progressBarSet):
            self.gds_info = info = obiGEO.GDSInfo()
        milestones = set(range(0, len(info), max(len(info)/100, 1)))
        self.cells = cells = []
        gdsLinks = []
        pmLinks = []
        localGDS = []
        full_text_search_data = []
        
        self.gds = []
        for i, (name, gds) in enumerate(info.items()):
            local = os.path.exists(orngServerFiles.localpath(obiGEO.DOMAIN, gds["dataset_id"] + ".soft.gz"))
            cells.append([" " if local else "", gds["dataset_id"], gds["title"], gds["platform_organism"], len(gds["samples"]), gds["feature_count"],
                          gds["gene_count"], len(gds["subsets"]), gds.get("pubmed_id", "")])

            gdsLinks.append("http://www.ncbi.nlm.nih.gov/sites/GDSbrowser?acc=%s" % gds["dataset_id"])
            pmLinks.append("http://www.ncbi.nlm.nih.gov/pubmed/%s" % gds.get("pubmed_id") if gds.get("pubmed_id") else QVariant())

            if local:
                localGDS.append(i)
            self.gds.append(gds)
            
            full_text_search_data.append(unicode(" | ".join([gds.get(key, "").lower() for key in self.searchKeys]), errors="ignore"))
            
            if i in milestones:
                self.progressBarSet(100.0*i/len(info))

        model = TreeModel(cells, ["", "ID", "Title", "Organism", "Samples", "Features", "Genes", "Subsets", "PubMedID"], self.treeWidget)
        model.setColumnLinks(1, gdsLinks)
        model.setColumnLinks(8, pmLinks)
        
        for i, text in enumerate(full_text_search_data):
            model.setData(model.index(i, 0), QVariant(text), TextFilterRole)
        
        proxyModel = MySortFilterProxyModel(self.treeWidget)
#        proxyModel = QSortFilterProxyModel(self.treeWidget)
        proxyModel.setSourceModel(model)
        proxyModel.setFilterKeyColumn(0)
        proxyModel.setFilterRole(TextFilterRole)
        proxyModel.setFilterCaseSensitivity(False)
        proxyModel.setFilterFixedString(self.filterString)
        self.treeWidget.setModel(proxyModel)
        self.connect(self.treeWidget.selectionModel(), SIGNAL("selectionChanged(QItemSelection , QItemSelection )"), self.updateSelection)
        filterItems = " ".join([self.gds[i][key] for i in range(len(self.gds)) for key in self.searchKeys])
        filterItems = reduce(lambda s, d: s.replace(d, " "),
                             [",", ".", ":", ";", "!", "?", "(", ")", "{", "}"
                              "[", "]", "_", "-", "+", "\\", "|", "/", "%", "#",
                              "@", "$", "^", "&", "*", "<", ">", "~", "`"],
                             filterItems.lower())
        filterItems = sorted(set(filterItems.split(" ")))
        filterItems = [item for item in filterItems if len(filterItems) > 3]
        self.filterLineEdit.setItems(filterItems)
        
        for i in range(8):
            self.treeWidget.resizeColumnToContents(i)
        self.treeWidget.setColumnWidth(1, min(self.treeWidget.columnWidth(1), 300))
        self.treeWidget.setColumnWidth(2, min(self.treeWidget.columnWidth(2), 200))
        self.progressBarFinished()

        if self.currentGds:
            gdss = [(i, model.data(model.index(i,1), Qt.DisplayRole)) for i in range(model.rowCount())]
            current = [i for i, variant in gdss if variant.isValid() and str(variant.toString()) == self.currentGds["dataset_id"]]
            if current:
                mapFromSource = self.treeWidget.model().mapFromSource
                self.treeWidget.selectionModel().select(mapFromSource(model.index(current[0], 0)), QItemSelectionModel.Select | QItemSelectionModel.Rows)
            
        self.updateInfo()

    def updateSelection(self, *args):
        current = self.treeWidget.selectedIndexes()
        mapToSource = self.treeWidget.model().mapToSource
        current = [mapToSource(index).row() for index in current]
        if current:
            self.currentGds = self.gds[current[0]]
            self.setAnnotations(self.currentGds)
            self.infoGDS.setText(self.currentGds.get("description", ""))
        else:
            self.currentGds = None
#        self.commitButton.setDisabled(not bool(self.currentGds))
        self.commitIf()
        
    
    def setAnnotations(self, gds):
        self._annotationsUpdating = True
        self.annotationsTree.clear()
        annotations = reduce(lambda d, info: d[info["type"]].add(info["description"]) or d, gds["subsets"], defaultdict(set))
        subsetscount = dict([(s["description"], str(len(s["sample_id"]))) for s in gds["subsets"]])
        for type, subsets in annotations.items():
            key = (gds["dataset_id"], type)
            subsetItem = QTreeWidgetItem(self.annotationsTree, [type])
            subsetItem.setFlags(subsetItem.flags() | Qt.ItemIsUserCheckable | Qt.ItemIsTristate)
            subsetItem.setCheckState(0, self.gdsSelectionStates.get(key, Qt.Checked))
            subsetItem.key = key
            for subset in subsets:
                key = (gds["dataset_id"], type, subset)
                item = QTreeWidgetItem(subsetItem, [subset, subsetscount.get(subset, "")])
                item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
                item.setCheckState(0, self.gdsSelectionStates.get(key, Qt.Checked))
                item.key = key
        self._annotationsUpdating = False
        self.annotationsTree.expandAll()
        for i in range(self.annotationsTree.columnCount()):
            self.annotationsTree.resizeColumnToContents(i)
                
    def annotationSelectionChanged(self, item, column):
        if self._annotationsUpdating:
            return 
        for i in range(self.annotationsTree.topLevelItemCount()):
            item = self.annotationsTree.topLevelItem(i)
            self.gdsSelectionStates[item.key] = item.checkState(0)
            for j in range(item.childCount()):
                child = item.child(j)
                self.gdsSelectionStates[child.key] = child.checkState(0)
        
    def rowFiltered(self, row):
        filterStrings = self.filterString.lower().split()
        try:
            string = " ".join([self.gds[row].get(key, "").lower() for key in self.searchKeys])
            return not all([s in string for s in filterStrings])
        except UnicodeDecodeError:
            string = " ".join([unicode(self.gds[row].get(key, "").lower(), errors="ignore") for key in self.searchKeys])
            return not all([s in string for s in filterStrings])
    
    def filter(self):
        filter_string = unicode(self.filterLineEdit.text(), errors="ignore")
        proxyModel = self.treeWidget.model()
        if proxyModel:
            strings = filter_string.lower().strip().split()
            proxyModel.setFilterFixedStrings(strings)
            self.updateInfo()

    def selectedSamples(self):
        """ Return the currently selected sample annotations (list of
        sample type, sample value tuples).
        
        .. note:: if some Sample annotation type has no selected values.
                  this method will return all values for it.
        
        """
        samples = []
        unused_types = []
        for stype in childiter(self.annotationsTree.invisibleRootItem()): 
            selected_values = []
            all_values = []
            for sval in childiter(stype):
                value = (str(stype.text(0)), str(sval.text(0)))
                if self.gdsSelectionStates.get(sval.key, True):
                    selected_values.append(value)
                all_values.append(value)
            if selected_values:
                samples.extend(selected_values)
            else:
                # If no sample of sample type is selected we don't filter on it.
                samples.extend(all_values)
                unused_types.append(str(stype.text(0)))
                
        return samples
    
    def commitIf(self):
        if self.autoCommit:
            self.commit()
        else:
            self.selectionChanged = True
    
    def commit(self):
        if self.currentGds:
            self.error(0) 
            sample_type = None
            self.progressBarInit()
            self.progressBarSet(10)
            
            def getdata(gds_id, **kwargs):
                gds = obiGEO.GDS(gds_id)
                data = gds.getdata(**kwargs)
                return data
            
            self.setEnabled(False)
            call = self.asyncCall(getdata, (self.currentGds["dataset_id"],), dict(report_genes=self.mergeSpots,
                                           transpose=self.outputRows,
                                           sample_type=sample_type if sample_type!="Include all" else None),
                                  onResult=self.onData,
                                  onFinished=lambda: self.setEnabled(True),
                                  onError=self.onAsyncError,
                                  threadPool=QThreadPool.globalInstance()
                                 )
            call.__call__() #invoke

    def onAsyncError(self, (exctype, value, tb)):
        import ftplib
        if issubclass(exctype, ftplib.error_temp):
            self.error(0, "Can not download dataset from NCBI ftp server! Try again later.")
        elif issubclass(exctype, ftplib.all_errors):
            self.error(0, "Error while connecting to the NCBI ftp server! %s" % str(value))
        else:
            sys.excepthook(exctype, value, tb)
            
        self.progressBarFinished()

    def onData(self, data):
        self.progressBarSet(50)
        
        samples = self.selectedSamples()
        
        self.warning(0)
        message = None
        if self.outputRows:
            samples = set(s[1] for s in samples) # dont have info on sample types in the data class variable
            select = [1 if samples.issuperset(str(ex.getclass()).split("|")) else 0 for ex in data]
            data = data.select(select)
            # TODO: add sample types as separate features  
            data.domain.classVar.values = ["|".join([cl for cl in val.split("|") if cl in samples]) for val in data.domain.classVar.values]
            if len(data) == 0:
                message = "No samples with selected sample annotations."
        else:
            samples = set(samples)
            domain = orange.Domain([attr for attr in data.domain.attributes if samples.issuperset(attr.attributes.items())], data.domain.classVar)
            domain.addmetas(data.domain.getmetas())
            if len(domain.attributes) == 0:
                message = "No samples with selected sample annotations."
            stypes = set(s[0] for s in samples)
            for attr in domain.attributes:
                attr.attributes = dict([(key, value) for key, value in attr.attributes.items() if key in stypes])
            data = orange.ExampleTable(domain, data)
        
        if message is not None:
            self.warning(0, message)
            
        data_hints.set_hint(data, "taxid", self.currentGds.get("taxid", ""), 10.0)
        data_hints.set_hint(data, "genesinrows", self.outputRows, 10.0)
        
        self.progressBarFinished()
        self.send("Expression Data", data)

        model = self.treeWidget.model().sourceModel()
        row = self.gds.index(self.currentGds)
#            model._roleData[Qt.ForegroundRole][row].update(zip(range(1, 7), [QVariant(QColor(LOCAL_GDS_COLOR))] * 6))
        model.setData(model.index(row, 0),  QVariant(" "), Qt.DisplayRole) 
#            model.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), model.index(row, 0), model.index(row, 0))
        self.updateInfo()
        self.selectionChanged = False
        
    def splitterMoved(self, *args):
        self.splitterSettings = [str(sp.saveState()) for sp in self.splitters]

if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWGEODatasets()
    w.show()
    app.exec_()
    w.saveSettings()



"""
<name>Quality Control</name>
<description>Experiment quality control</description>

"""

from __future__ import absolute_import, with_statement

import sys
from collections import defaultdict
from contextlib import contextmanager
from pprint import pprint

import numpy

import Orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *
from Orange.OrangeWidgets.OWItemModels import PyListModel, safe_text
from Orange.OrangeWidgets.OWGraphics import GraphicsSimpleTextLayoutItem

from .. import obiExperiments as exp

from .OWGenotypeDistances import SetContextHandler

DEBUG = False

@contextmanager
def widget_disable(widget):
    """A context to disable the widget (enabled property)  
    """
    widget.setEnabled(False)
    try:
        yield
    finally:
        widget.setEnabled(True)
    
    
@contextmanager
def disable_updates(widget):
    """
    A context that sets '_disable_updates' member to True, and
    then restores it.
    
    """
    widget._disable_updates = True
    try:
        yield
    finally:
        widget._disable_updates = False


def group_label(splits, groups):
    """Return group label.
    """
    labels = ["%s=%s" % (split, group) \
              for split, group in zip(splits, groups)]
    return " | ".join(labels)


def sort_label(sort, attr):
    """Return within group sorted items label for attribute.
    """
    items = [(key, attr.attributes.get(key, "?")) \
             for key in sort]
    labels = ["%s=%s" % tuple(item) for item in items]
    return " | ".join(labels)


def float_if_posible(val):
    """Return val as float if possible otherwise return the value unchanged.
    
    """
    try:
        return float(val)
    except ValueError:
        return val
    
    
def experiment_description(feature):
    """Return experiment description from ``feature.attributes``. 
    """
    text = ""
    if feature.attributes:
        items = feature.attributes.items()
        items = [(safe_text(key), safe_text(value)) for key, value in items]
        labels = map("%s = %s".__mod__, items)
        text += "<b>%s</b><br/>" % safe_text(feature.name)
        text += "<br/>".join(labels)
    return text


class OWQualityControl(OWWidget):
    contextHandlers = {"": SetContextHandler("")}
    settingsList = ["selected_distance_index"]
    
    DISTANCE_FUNCTIONS = [("Distance from Pearson correlation",
                           exp.dist_pcorr),
                          ("Euclidean distance", 
                           exp.dist_eucl),
                          ("Distance from Spearman correlation", 
                           exp.dist_spearman)]

    def __init__(self, parent=None, signalManager=None,
                 title="Quality Control"):
        OWWidget.__init__(self, parent, signalManager, title,
                          wantGraph=True)

        self.inputs = [("Experiment Data", Orange.data.Table, self.set_data)]

        ## Settings
        self.selected_distance_index = 0

        ## Attributes
        self.data = None
        self.distances = None
        self.groups = None
        self.unique_pos = None
        self.base_group_index = 0

        ## GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.info_box = OWGUI.widgetLabel(box, "\n")

        ## Separate By box
        box = OWGUI.widgetBox(self.controlArea, "Separate By")
        self.split_by_model = PyListModel()
        self.split_by_view = QListView()
        self.split_by_view.setSelectionMode(QListView.ExtendedSelection)
        self.split_by_view.setModel(self.split_by_model)
        box.layout().addWidget(self.split_by_view)

        self.connect(self.split_by_view.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.on_split_key_changed)

        ## Sort By box
        box = OWGUI.widgetBox(self.controlArea, "Sort By")
        self.sort_by_model = PyListModel()
        self.sort_by_view = QListView()
        self.sort_by_view.setSelectionMode(QListView.ExtendedSelection)
        self.sort_by_view.setModel(self.sort_by_model)
        box.layout().addWidget(self.sort_by_view)
        
        self.connect(self.sort_by_view.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.on_sort_key_changed)

        ## Distance box
        box = OWGUI.widgetBox(self.controlArea, "Distance Measure")
        OWGUI.comboBox(box, self, "selected_distance_index",
                       items=[t[0] for t in self.DISTANCE_FUNCTIONS],
                       callback=self.on_distance_measure_changed)

        self.connect(self.graphButton,
                     SIGNAL("clicked()"),
                     self.save_graph)
        
        self.scene = QGraphicsScene()
        self.scene_view = QualityGraphicsView(self.scene)
        self.scene_view.setRenderHints(QPainter.Antialiasing)
        self.scene_view.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        self.mainArea.layout().addWidget(self.scene_view)
        
        self.connect(self.scene_view,
                     SIGNAL("view_size_changed(QSize)"),
                     self.on_view_resize)

        self._disable_updates = False
        self._cached_distances = {}
        self._base_index_hints = {}
        self.main_widget = None
        
        self.resize(800, 600)

    def clear(self):
        """Clear the widget state.
        """
        self.data = None
        self.distances = None
        self.groups = None
        self.unique_pos = None
        
        with disable_updates(self):
            self.split_by_model[:] = []
            self.sort_by_model[:] = []
        
        self.main_widget = None
        self.scene.clear()
        self.info_box.setText("\n")
        self._cached_distances = {}

    def set_data(self, data=None):
        """Set input experiment data.
        """
        self.clear()
        
        self.error(0)
        self.warning(0)
        
        if data is not None:
            keys = self.get_suitable_keys(data)
            if not keys:
                self.error(0, "Data has no suitable feature labels.")
                data = None
                
        self.data = data

    def handleNewSignals(self):
        """Called after all signals have been set. 
        """
        if self.data:
            self.on_new_data()
        else:
            self.closeContext("")
            self.clear()

    def update_label_candidates(self):
        """Update the label candidates selection GUI 
        (Group/Sort By views).
        
        """
        keys = self.get_suitable_keys(self.data)
        with disable_updates(self):
            self.split_by_model[:] = keys
            self.sort_by_model[:] = keys
        
    def get_suitable_keys(self, data):
        """ Return suitable attr label keys from the data where
        the key has at least two unique values in the data.
        
        """
        attrs = [attr.attributes.items() for attr in data.domain.attributes]
        attrs  = reduce(list.__add__, attrs, [])
        # in case someone put non string values in attributes dict
        attrs = [(str(key), str(value)) for key, value in attrs]
        attrs = set(attrs)
        values = defaultdict(set)
        for key, value in attrs:
            values[key].add(value)
        keys = [key for key in values if len(values[key]) > 1]
        return keys

    def selected_split_by_labels(self):
        """Return the current selected split labels.
        """
        sel_m = self.split_by_view.selectionModel()
        indices = [r.row() for r in sel_m.selectedRows()]
        return [self.sort_by_model[i] for i in indices]

    def selected_sort_by_labels(self):
        """Return the current selected sort labels
        """
        sel_m = self.sort_by_view.selectionModel()
        indices = [r.row() for r in sel_m.selectedRows()]
        return [self.sort_by_model[i] for i in indices]

    def selected_distance(self):
        """Return the selected distance function.
        """
        return self.DISTANCE_FUNCTIONS[self.selected_distance_index][1]
    
    def selected_base_group_index(self):
        """Return the selected base group index
        """
        return self.base_group_index
    
    def selected_base_indices(self, base_group_index=None):
        indices = []
        for g, ind in self.groups:
            if base_group_index is None:
                label = group_label(self.selected_split_by_labels(), g)
                ind = [i for i in ind if i is not None]
                i = self._base_index_hints.get(label, ind[0] if ind else None)
            else:
                i = ind[base_group_index]
            indices.append(i)
        return indices

    def on_new_data(self):
        """We have new data and need to recompute all.
        """
        self.closeContext("")
        
        self.update_label_candidates()
        self.info_box.setText("%s genes \n%s experiments" % (
                                len(self.data), 
                                len(self.data.domain.attributes)
                                )
                              )
        
        self.base_group_index = 0
        
        keys = self.get_suitable_keys(self.data)
        self.openContext("", keys)
        
        ## Restore saved context
        context = self.currentContexts[""]
        split_by_labels = getattr(context, "split_by_labels", set())
        sort_by_labels = getattr(context, "sort_by_labels", set())
        
        def select(model, selection_model, selected_items):
            """Select items in a Qt item model view
            """
            all_items = list(model)
            try:
                indices = [all_items.index(item) for item in selected_items]
            except:
                indices = []
            for ind in indices:
                selection_model.select(model.index(ind), 
                                       QItemSelectionModel.Select)
                
        with disable_updates(self):
            select(self.split_by_view.model(),
                   self.split_by_view.selectionModel(),
                   split_by_labels)
            
            select(self.sort_by_view.model(),
                   self.sort_by_view.selectionModel(),
                   sort_by_labels)
        
        with widget_disable(self):
            self.split_and_update()
        
    def on_split_key_changed(self, *args):
        """Split key has changed
        """
        with widget_disable(self):
            if not self._disable_updates:
                self.base_group_index = 0
                context = self.currentContexts[""]
                context.split_by_labels = self.selected_split_by_labels()
                self.split_and_update()
    
    def on_sort_key_changed(self, *args):
        """Sort key has changed
        """
        with widget_disable(self):
            if not self._disable_updates:
                self.base_group_index = 0
                context = self.currentContexts[""]
                context.sort_by_labels = self.selected_sort_by_labels()
                self.split_and_update()
        
    def on_distance_measure_changed(self):
        """Distance measure has changed
        """
        if self.data is not None:
            with widget_disable(self):
                self.update_distances()
                self.replot_experiments()
        
    def on_view_resize(self, size):
        """The view with the quality plot has changed
        """
        if self.main_widget:
            current = self.main_widget.size()
            self.main_widget.resize(size.width() - 6, 
                                    current.height())
            
            self.scene.setSceneRect(self.scene.itemsBoundingRect())
        
    def on_rug_item_clicked(self, item):
        """An ``item`` in the quality plot has been clicked.
        """
        update = False
        sort_by_labels = self.selected_sort_by_labels()
        if sort_by_labels and item.in_group:
            ## The item is part of the group
            if item.group_index != self.base_group_index:
                self.base_group_index = item.group_index
                update = True
            
        else:
            if sort_by_labels:
                # If the user clicked on an background item it
                # invalidates the sorted labels selection
                with disable_updates(self):
                    self.sort_by_view.selectionModel().clear()
                    update = True
                    
            index = item.index
            group = item.group
            label = group_label(self.selected_split_by_labels(), group)
            
            if self._base_index_hints.get(label, 0) != index:
                self._base_index_hints[label] = index
                update = True
            
        if update:
            with widget_disable(self):
                self.split_and_update()
        
    def split_and_update(self):
        """
        Split the data based on the selected sort/split labels
        and update the quality plot.
        
        """
        split_labels = self.selected_split_by_labels()
        sort_labels = self.selected_sort_by_labels()
        
        self.warning(0)
        if not split_labels:
            self.warning(0, "No separate by label selected.")
            
        self.groups, self.unique_pos = \
                exp.separate_by(self.data, split_labels,
                                consider=sort_labels,
                                add_empty=True)
        
        
        self.groups = sorted(self.groups.items(),
                             key=lambda t: map(float_if_posible, t[0]))
        self.unique_pos = sorted(self.unique_pos.items(),
                                 key=lambda t: map(float_if_posible, t[0]))
        
        
        if self.groups:
            if sort_labels:
                group_base = self.selected_base_group_index()
                base_indices = self.selected_base_indices(group_base)
            else:
                base_indices = self.selected_base_indices()
            self.update_distances(base_indices)
            self.replot_experiments()

    def get_cached_distances(self, measure):
        if measure not in self._cached_distances:
            attrs = self.data.domain.attributes
            mat = Orange.misc.SymMatrix(len(attrs))
            self._cached_distances[measure] = \
                (mat, set(zip(range(len(attrs)), range(len(attrs)))))
            
        return self._cached_distances[measure]
        
    def get_cached_distance(self, measure, i, j):
        matrix, computed = self.get_cached_distances(measure)
        key = (i, j) if i < j else (j, i) 
        if key in computed:
            return matrix[i, j]
        else:
            return None
        
    def get_distance(self, measure, i, j):
        d = self.get_cached_distance(measure, i, j)
        if d is None:
            vec_i = exp.linearize(self.data, [i])
            vec_j = exp.linearize(self.data, [j])
            d = distance(vec_i, vec_j)
            mat, computed = self.get_cached_distances(measure)
            mat[i, j] = d
            key = key = (i, j) if i < j else (j, i)
            computed.add(key)
        return d
    
    def store_distance(self, measure, i, j, dist):
        matrix, computed = self.get_cached_distances(measure)
        key = key = (i, j) if i < j else (j, i)
        matrix[i, j] = dist
        computed.add(key)
        
    def update_distances(self, base_indices=()):
        """Recompute the experiment distances.
        """
        distance = self.selected_distance()
        if base_indices == ():
            base_group_index = self.selected_base_group_index()
            base_indices = [ind[base_group_index] \
                            for _, ind in self.groups]
            
        assert(len(base_indices) == len(self.groups)) 
        
        base_distances = []
        attributes = self.data.domain.attributes
        pb = OWGUI.ProgressBar(self, len(self.groups) * \
                               len(attributes))
        
        cached_distances, filled_set = self.get_cached_distances(distance)
        
        for (group, indices), base_index in zip(self.groups, base_indices):
            # Base column of the group
            if base_index is not None:
                base_vec = exp.linearize(self.data, [base_index])
                distances = []
                # Compute the distances between base column 
                # and all the rest data columns.
                for i in range(len(attributes)):
                    if i == base_index:
                        distances.append(0.0)
                    elif self.get_cached_distance(distance, i, base_index) is not None:
                        distances.append(self.get_cached_distance(distance, i, base_index))
                    else:
                        vec_i = exp.linearize(self.data, [i])
                        dist = distance(base_vec, vec_i)
                        self.store_distance(distance, i, base_index, dist)
                        distances.append(dist)
                    pb.advance()
                    
                base_distances.append(distances)
            else:
                base_distances.append(None)
                
        pb.finish()
        self.distances = base_distances

    def replot_experiments(self):
        """Replot the whole quality plot.
        """
        self.scene.clear()
        labels = []
        
        max_dist = numpy.max(filter(None, self.distances))
        rug_widgets = []
        
        group_pen = QPen(QColor(0, 0, 0))
        group_pen.setWidth(2)
        group_pen.setCapStyle(Qt.RoundCap)
        background_pen = QPen(QColor(0, 0, 250, 150))
        background_pen.setWidth(1)
        background_pen.setCapStyle(Qt.RoundCap)
        
        main_widget = QualityControlWidget()
        layout = QGraphicsGridLayout()
        split_by = self.selected_split_by_labels()
        sort_by = self.selected_sort_by_labels()
        attributes = self.data.domain.attributes
        if self.data:
            for (group, indices), dist_vec in zip(self.groups, self.distances):
                indices_set = set(indices)
                rug_items = []
                if dist_vec is not None:
                    for i, attr in enumerate(attributes):
                        # Is this a within group distance or background
                        in_group = i in indices_set 
                        if in_group:
                            rug_item = ClickableRugItem(dist_vec[i] / max_dist,
                                           1.0, self.on_rug_item_clicked)
                            rug_item.setPen(group_pen)
                            tooltip = experiment_description(attr)
                            rug_item.setToolTip(tooltip)
                            rug_item.group_index = indices.index(i)
                            rug_item.setZValue(rug_item.zValue() + 1)
                        else:
                            rug_item = ClickableRugItem(dist_vec[i] / max_dist,
                                           0.85, self.on_rug_item_clicked)
                            rug_item.setPen(background_pen)
                            tooltip = experiment_description(attr)
                            rug_item.setToolTip(tooltip)
                            
                        rug_item.group = group
                        rug_item.index = i
                        rug_item.in_group = in_group
                        
                        rug_items.append(rug_item)
                    
                rug_widget = RugGraphicsWidget()
                rug_widget.set_rug(rug_items)
                
                rug_widgets.append(rug_widget)
                
                label = group_label(self.selected_split_by_labels(), group)
                label_item = QGraphicsSimpleTextItem(label, main_widget)
                label_item = GraphicsSimpleTextLayoutItem(label_item)
                label_item.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
                labels.append(label_item)
        
        for i, (label, w) in enumerate(zip(labels, rug_widgets)):
            layout.addItem(label, i, 0, Qt.AlignVCenter)
            layout.addItem(w, i, 1)
            layout.setRowMaximumHeight(i, 30)
            
        main_widget.setLayout(layout)
        self.scene.addItem(main_widget)
        main_widget.show()
        self.main_widget = main_widget
        self.rug_widgets = rug_widgets
        self.labels = labels
        self.on_view_resize(self.scene_view.size())
        
    def save_graph(self):
        from Orange.OrangeWidgets.OWDlgs import OWChooseImageSizeDlg
        dlg = OWChooseImageSizeDlg(self.scene, parent=self)
        dlg.exec_()


class RugGraphicsWidget(QGraphicsWidget):
    def __init__(self, parent=None, rug=None):
        QGraphicsWidget.__init__(self, parent)
        self.rug_items = []
        self.set_rug(rug)
        self.setMaximumHeight(30)
        self.setMinimumHeight(30)
        
    def clear(self):
        """
        Clear all rug items from this widget and remove them 
        from the scene.
         
        """
        for item in self.rug_items:
            item.setParent(None)
            if self.scene() is not None:
                self.scene().removeItem(item)

    def set_rug(self, rug):
        """
        Set the rug items.
        
        ``rug`` must be a list of floats or already initialized
        instances of RugItem. The widget takes ownership of all 
        items.
          
        """
        rug = rug if rug is not None else []
        self.clear()
        self.add_rug(rug)

    def add_rug(self, rug):
        """
        Add rug items.
        
        See :obj:`set_rug`
        
        """
        items = []
        for item in rug:
            if isinstance(item, float):
                item = RugItem(value=item)
                items.append(item)
            elif isinstance(item, RugItem):
                items.append(item)

        for item in items:
            item.setParentItem(self)

        self.rug_items.extend(items)

        self.update_rug_geometry()
        
    def update_rug_geometry(self):
        """Recompute the rug items positions within this widget.
        """
        size = self.size()
        height = size.height()
        width = size.width()
        
        for item in self.rug_items:
            offset = (1.0 - item.height) * height / 2.0
            item.setPos(width * item.value, 0)
            item.setLine(0., offset, 0., height - offset)

    def resizeEvent(self, event):
        """Reimplemented from QGraphicsWidget
        """
        QGraphicsWidget.resizeEvent(self, event)
        self.update_rug_geometry()

    def setGeometry(self, geom):
        """Reimplemented from QGraphicsWidget
        """
        QGraphicsWidget.setGeometry(self, geom)


class RugItem(QGraphicsLineItem):
    def __init__(self, value, height):
        QGraphicsLineItem.__init__(self)
        self.value = value
        self.height = height

    def set_height(self, height):
        """Set the height of this item (in ratio of the rug height)
        """
        self.height = height
        
class ClickableRugItem(RugItem):
    def __init__(self, value, height, on_pressed):
        RugItem.__init__(self, value, height)
        self.on_pressed = on_pressed
        self.setAcceptedMouseButtons(Qt.LeftButton)
        self.setAcceptHoverEvents(True)
        
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton and self.on_pressed:
            self.on_pressed(self)
            
    def hoverEnterEvent(self, event):
        pen = QPen(self.pen())
        pen.setWidthF(3)
        self.setPen(pen)
        return RugItem.hoverEnterEvent(self, event)
    
    def hoverLeaveEvent(self, event):
        pen = QPen(self.pen())
        pen.setWidth(2)
        self.setPen(pen)
        return RugItem.hoverLeaveEvent(self, event)


class QualityGraphicsView(QGraphicsView):
    def resizeEvent(self, event):
        QGraphicsView.resizeEvent(self, event)
        self.emit(SIGNAL("view_size_changed(QSize)"),
                  event.size())


class QualityControlWidget(QGraphicsWidget):
    if DEBUG:
        def paint(self, painter, options, widget=0):
            rect =  self.geometry()
            rect.translate(-self.pos())
            painter.drawRect(rect)
            
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWQualityControl()
#    data = Orange.data.Table("doc:dicty-abc-sample.tab")
    data = Orange.data.Table("doc:pipa.tab")

    w.set_data(data)
    w.show()
    w.handleNewSignals()
    app.exec_()
    w.set_data(None)
    w.handleNewSignals()
    w.saveSettings()
